package com.gitlink.softbot.controller.market;


import com.gitlink.softbot.global.exception.BotException;
import com.gitlink.softbot.global.vo.Result;
import com.gitlink.softbot.service.market.IMarketService;
import com.gitlink.softbot.vo.GetAllBotCategory;
import com.gitlink.softbot.vo.GetBotDetailResponse;
import com.gitlink.softbot.vo.MarketBotPagesVO;
import com.gitlink.softbot.vo.MarketBotPagesVO1;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;


@RestController
@CrossOrigin()
public class MarketController {

    @Resource
    IMarketService marketService;


    /***
     * 通过功能类型查询MarketBot
     * @param func
     * @param pageNo
     * @param pageSize
     * @return
     * @throws BotException
     */
    @GetMapping ("/getMarketBotsByFunc")//当func为空则返回所有
    public Result<MarketBotPagesVO> getMarketBot(@RequestParam String func, @RequestParam(value = "page_no",defaultValue = "1",required = false) Integer pageNo,
                                                 @RequestParam(value = "page_size",defaultValue = "12",required = false) Integer pageSize) throws BotException {
        MarketBotPagesVO marketBotPagesVO = marketService.getMarketBotByCategory(func,pageNo,pageSize);
       Result<MarketBotPagesVO> result = new Result<>();
       return result.build(marketBotPagesVO).build(200).build("success!");
    }

    /**
     * 模糊查询MarketBot
     * @param keyword
     * @param pageIndex
     * @param pageSize
     * @return
     * @throws BotException
     */
    @GetMapping("/getContentsLike")
    public Result<MarketBotPagesVO> getMarketBotsLike(@RequestParam("keyword") String keyword,@RequestParam(value = "page_no",defaultValue = "1",required = false) Integer pageIndex,
                                                      @RequestParam(value = "page_size",defaultValue = "12",required = false) Integer pageSize) throws BotException{
        MarketBotPagesVO marketBotPagesVO = marketService.getMarketBotByNameLike(keyword,pageIndex,pageSize);
        Result<MarketBotPagesVO> result = new Result<>();
        return result.build(marketBotPagesVO).build(200).build("success!");
    }

    /**
     * 获取bot主页详情
     * @param botId
     * @param userId
     * @return
     */
    @GetMapping("/getBotDetail")
    public Result<?> getBotDetail(@RequestParam(value = "bot_id",required = true) Integer botId,@RequestParam(value = "user_id",required = false) Integer userId){

        GetBotDetailResponse getBotDetailResponse = marketService.getBotDetail(userId,botId);
        Result<GetBotDetailResponse> result = new Result<>();
        return result.build(getBotDetailResponse).build(200).build("success!");

    }

    /**
     * 获取所有Bot种类
     * @return
     */
    @GetMapping("/getAllBotCategory")
    public Result<?> getAllBotCategory(){
        GetAllBotCategory getAllBotCategory = marketService.getAllBotCategory();
        Result<GetAllBotCategory> result = new Result<>();
        return result.build(getAllBotCategory).build(200).build("success!");
    }

    /**
     *
     * @param keyword
     * @param func
     * @param pageIndex
     * @param pageSize
     * @return
     * @throws BotException
     */
    @GetMapping("/getContentsLikeNameAndFunc")
    public Result<MarketBotPagesVO1> getContentsLikeNameAndFunc(@RequestParam(value = "name",required = false) String keyword,
                                                                @RequestParam(value= "func", required = false) String func,
                                                                @RequestParam(value = "page_no",defaultValue = "1",required = false) Integer pageIndex,
                                                                @RequestParam(value = "page_size",defaultValue = "12",required = false) Integer pageSize) throws BotException{
        MarketBotPagesVO1 marketBotPagesVO1 = marketService.getContentsLikeNameAndFunc(keyword,func,pageIndex,pageSize);
        Result<MarketBotPagesVO1> result = new Result<>();
        return result.build(marketBotPagesVO1).build(200).build("success!");
    }


}
