package com.gitlink.softbot.controller.user;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.gitlink.softbot.annotation.NoRepeatSubmit;
import com.gitlink.softbot.global.exception.BotException;
import com.gitlink.softbot.global.vo.Response;
import com.gitlink.softbot.global.vo.Result;
import com.gitlink.softbot.service.user.IUserService;
import com.gitlink.softbot.utils.AuthOperate;
import com.gitlink.softbot.utils.GitLinkApi;
import com.gitlink.softbot.vo.*;

import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;
import com.gitlink.softbot.vo.BotInputVO;
import javax.annotation.Resource;
import javax.validation.Valid;


@RestController
@CrossOrigin()
public class UserController {


    /**
     * 注册Bot
     */
    @Resource
    IUserService userService;

    @Resource
    GitLinkApi api;
    @PostMapping("/registerBot")
    public Result<BotOutputVO> registerBot(@Valid @RequestBody BotInputVO botInputVO ) throws BotException {
        Result<BotOutputVO> result  = new Result<>();
        BotOutputVO botOutputVO = userService.registerBot(botInputVO);
        return result.build(botOutputVO).build(200).build("register success!");
    }
    /**
     * 配置注册的Bot
     * @param botInputVO
     * @return
     * @throws BotException
     */
    @PostMapping("/registerUpdateBot")
    public Result<BotOutputVO> updateBot(@Valid @RequestBody BotInputVO botInputVO) throws BotException{
        if (botInputVO.getBotId()==null){
            throw new BotException("botId is null");
        }
        Result<BotOutputVO> result = new Result<>();
        BotOutputVO botOutputVO = userService.updateBot(botInputVO);
        return result.build(botOutputVO).build(200).build("update success!");
    }

    /**
     * 获取个人注册Bots
     * @param userId
     * @return
     */
    @GetMapping("/getMyBot")
    public Result<GetBotResponse> getMyBots(@RequestParam("user_id") Integer userId){
        Result<GetBotResponse> result = new Result<>();
        GetBotResponse getBotResponse = userService.getBots(userId);
        return result.build(getBotResponse).build("getBots success!").build(200);
    }

    /**
     * 获取个人注册的bot
     */
    @GetMapping("/getRegisterBot")
    public Result<GetRegisterBotResponse> getRegisterBot(@RequestParam("user_id") Integer userId,@RequestParam("bot_id") Integer botId) throws BotException {
        Result<GetRegisterBotResponse> result = new Result<>();
        GetRegisterBotResponse getRegisterBotResponse = userService.getRegisterBot(userId,botId);
        return  result.build(200)
                .build("success")
                .build(getRegisterBotResponse);
    }

    /**
     * 删除注册的Bot
     * @param deleteBotRequest
     * @return
     */
    @PostMapping("/deleteBot")
    public Result<?> deleteBot(@Valid @RequestBody DeleteBotRequest deleteBotRequest) throws BotException {
        Result<?> result = new Result<>();
        userService.deleteBot(deleteBotRequest);
        return result.build(200).build("delete success!");
    }

    /**
     * 上市Bot
     * @param botToMarketRequest
     * @return
     * @throws BotException
     */
    @PostMapping("/marketBot")
    public Result<?> marketBot(@Valid @RequestBody BotToMarketRequest botToMarketRequest) throws BotException{
        Result<?> result = new Result<>();
        userService.botToMarket(botToMarketRequest);
        return result.build(200).build("market success!");
    }


    /**
     * 返回所有上市的MarketBot
     * @param userId
     * @return
     */
    @GetMapping("/myMarketBots")
    public Result<GetMarketBotsResponse> getMyMarketBots(@RequestParam("user_id") Integer userId){
        Result<GetMarketBotsResponse> result = new Result<>();
        GetMarketBotsRequest getMarketBotsRequest = new GetMarketBotsRequest();
        getMarketBotsRequest.setUserId(userId);
        GetMarketBotsResponse getMarketBotsResponse = userService.getMarketBots(getMarketBotsRequest);
        return result.build(getMarketBotsResponse).build(200).build("getMarketBots success!");
    }

    /**
     * 通过BotId查询MarketBot
     * @param botId
     * @return
     */
    @GetMapping("/getMarketBotById")
    public Result<MarketBotVO> getMarketBotById(@RequestParam("bot_id") Integer botId) throws BotException{
        Result<MarketBotVO> result = new Result<>();
        MarketBotVO marketBotVO = userService.getMarketBotById(botId);
        return result.build(marketBotVO).build(200).build("get botById success!");
    }

    /**
     * 更新上市bot信息
     * @param botToMarketRequest
     * @return
     * @throws BotException
     */
    @PostMapping("/updateMarketBot")
    public Result<?> updateMarketBot(@Valid @RequestBody BotToMarketRequest botToMarketRequest )throws BotException{
        Result<?> result = new Result<>();
        userService.updateMarketBot(botToMarketRequest);
        return result.build(200).build("updateBot success!");
    }

    /**
     * bot下架
     * @param botId
     * @return
     * @throws BotException
     */
    @PostMapping("/downMarket")
    public Result<?> downMarket(@RequestParam("id") Integer botId) throws BotException{
        Result<?> result = new Result<>();
        userService.downMarketBot(botId);
        return result.build(200).build("downMarketBot success!");
    }
    /**
     * 用户安装bot
     * @param installMarketBotRequest
     * @return
     */
    @PostMapping("installMarketBot")
    public Result<?> installMarketBot(@RequestBody InstallMarketBotRequest installMarketBotRequest) throws BotException, JsonProcessingException {
        Result<?> result = new Result<>();
        userService.installMarketBot(installMarketBotRequest);
        return result.build(200).build("install bot success!");
    }

    /**
     * 用户卸载某个仓库的所有bot
     * @param unInstallMarketBotRequest
     * @return
     */
    @PostMapping("uninstallMarketBot")
    public Result<?> uninstallMarketBot(@RequestBody UnInstallMarketBotRequest unInstallMarketBotRequest) throws BotException{
        Result<?> result = new Result<>();
        userService.unInstallMarketBot(unInstallMarketBotRequest);
        return result.build(200).build("uninstall bot success!");
    }

    /**
     * 配置安装的Bot
     * @param updateInstallBotRequest
     * @return
     */
    @PostMapping("/updateInstallBot")
    @NoRepeatSubmit
    public Result<?> updateInstallBot(@Valid @RequestBody UpdateInstallBotRequest updateInstallBotRequest) throws Exception{
        Result<?> result = new Result<>();
        userService.updateInstallBot(updateInstallBotRequest);
        return result.build(200).build("update bot success!");
    }

    /**
     * 删除安装的Bot
     * @param deleteInstallBotRequest
     * @return
     */
    @PostMapping("/deleteInstallBot")
    public Result<?> deleteInstallBot(@Valid @RequestBody DeleteInstallBotRequest deleteInstallBotRequest) throws BotException{
        Result<?> result = new Result<>();
        userService.deleteInstallBot(deleteInstallBotRequest);
        return result.build(200).build("delete bot success!");
    }

    /**
     * 获取已经安装的Bot
     * @param userId
     * @return
     */
    @GetMapping("/getInstallBot")
    public Result<?> getInstallBot(@RequestParam("user_id") Integer userId, @RequestParam("bot_id") Integer botId,@RequestParam("login") String login, @RequestParam("repoIds") String repoIds) throws BotException{
        GetInstallBotRequest getInstallBotRequest = new GetInstallBotRequest(userId, login, botId , repoIds);
        Result<?> result = new Result<>();
        GetInstallBotResponse getInstallBotResponse = userService.getInstallBot(getInstallBotRequest);
        return result.build(200).build(getInstallBotResponse).build("get installBot success!");
    }

    /**
     * 转让Bot
     * @param transferBotRequest
     * @return
     */
    @PostMapping("/transferBot")
    public Result<?> transferBot(@RequestBody TransferBotRequest transferBotRequest) throws BotException {
        Result<?> result = new Result<>();
        userService.transferBot(transferBotRequest);
        return  result.build(200).build("transfer bot success!");
    }

    /**
     * 查看转让者该Bot转让状态
     * @param userId
     * @return
     */
    @GetMapping("/getTransferBotState")
    public Result<GetTransferFromBotResponse> getTransferBotState(@RequestParam("user_id") Integer userId, @RequestParam("bot_id") Integer botId,@RequestParam("login") String login){
        Result<GetTransferFromBotResponse> result = new Result<>();
        GetTransferFromBotRequest getTransferFromBotRequest = new GetTransferFromBotRequest(userId,login,botId);
        GetTransferFromBotResponse getTransferFromBotResponse = userService.getTransferBotState(getTransferFromBotRequest);
        return result.build(200).build("get state success!").build(getTransferFromBotResponse);
    }

    /**
     * 取消正在转让的Bot
     * @param cancelTransferBotRequest
     * @return
     * @throws BotException
     */
    @PostMapping("/cancelTransferBot")
    public Result<?> cancelTransferBot(@Valid @RequestBody CancelTransferBotRequest cancelTransferBotRequest) throws BotException{
        Result<?> result = new Result<>();
        userService.cancelTransferBot(cancelTransferBotRequest);
        return result.build(200).build("cancel success!");
    }

    /**
     * 待转让者查看所有待转让的Bot
     * @param userId
     * @return
     */
    @GetMapping("/getTransferToBot")
    public Result<GetTransferToBotResponse> getTransferToBotResponseResult(@RequestParam("user_id") Integer userId,
                                                                           @RequestParam("state") Integer state,
    @RequestParam("login") String login){
        GetTransferToBotRequest getTransferToBotRequest = new GetTransferToBotRequest(userId, login,state);
        Result<GetTransferToBotResponse> result = new Result<>();
        GetTransferToBotResponse getTransferToBotResponse = userService.getTransferToBot(getTransferToBotRequest);
        return result.build(200).build("get transferToBot success!").build(getTransferToBotResponse);
    }

    /**
     * 接收转让Bot
     * @param receiveTransferBotRequest
     * @return
     */
    @PostMapping("/receiveTransferBot")
    public Result<?> receiveTransferBot(@Valid @RequestBody ReceiveTransferBotRequest receiveTransferBotRequest) throws BotException {
        Result<?> result = new Result<>();
        userService.receiveTransferBot(receiveTransferBotRequest);
        return result.build(200).build("receive success!");
    }

    /**
     * 拒绝接收转让的Bot
     * @param refuseTransferBotRequest
     * @return
     */
    @PostMapping("refuseTransferBot")
    public Result<?> refuseTransferBot(@Valid @RequestBody RefuseTransferBotRequest refuseTransferBotRequest)  throws BotException{
        Result<?> result = new Result<>();
        userService.refuseTransferBot(refuseTransferBotRequest);
        return result.build(200).build("refuse transferBot success!");
    }

    /**
     * 判断是否安装该bot
     * @param repoIds
     * @param botId
     * @return
     */
    @GetMapping("/judgeIsIntallBot")
    public Result<?> judgeIsInstallBot(@RequestParam("repoIds") String repoIds, @RequestParam("bot_id") Integer botId){

        JudgeIsInstallBotResponse judgeIsInstallBotResponse = userService.judgeIsInstallBot(repoIds,botId);
        Result<JudgeIsInstallBotResponse> result = new Result<>();
        return result.build(judgeIsInstallBotResponse).build(200).build("success!");
    }

    /**
     * 用户获取所有安装的bot信息
     * @param userId
     * @return
     */
    @GetMapping("/getAllInstallBots")
    public Result<?> getAllInstallBots(@RequestParam("user_id") Integer userId, @RequestParam(value = "repoIds", required = false) String repoIds){
        Result<GetAllInstallBotsResponse> result = new Result<>();
        if(userId == null && (repoIds == null || "".equals(repoIds))) {
            return result.build("传入参数不能全空").build(500).build("fail!");
        }
        GetAllInstallBotsResponse getAllInstallBotsResponse = userService.getAllInstallBots(userId, repoIds);
        return result.
                build(getAllInstallBotsResponse).
                build(200).
                build("success!");
    }


    /**
     * 仓库获取所有安装的bot信息
     * @param storeId
     * @return
     */
    @GetMapping("/getStoreAllInstallBots")
    public Result<?> getStoreAllInstalledBots(@RequestParam("store_id") Integer storeId){
        GetStoreAllInstallBotResponse getStoreAllInstallBotResponse = userService.getStoreAllInstallBots(storeId);
        Result<GetStoreAllInstallBotResponse> result = new Result<>();
        return result.build(getStoreAllInstallBotResponse).build(200).build("success!");
    }
    /**
     * 更新ClientSecret
     */
    @PostMapping("/updateClientSecret")
    public Result<?> updateClientSecret(@RequestParam("user_id") Integer userId,@RequestParam("bot_id") Integer botId){

        Response response = api.activeBotAuth(userId,botId, AuthOperate.UPDATE_SECRET);
        Result<Object> result = new Result<>();
        return result.build(response).build(200).build("update success!");
    }


}
