package com.gitlink.softbot.service.market.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gitlink.softbot.dao.db.BotCategoryMapper;
import com.gitlink.softbot.dao.db.BotMapper;
import com.gitlink.softbot.dao.db.MarketBotMapper;
import com.gitlink.softbot.dao.db.RegisterMapper;
import com.gitlink.softbot.dao.db.BotLimitMapper;
import com.gitlink.softbot.entity.db.*;
import com.gitlink.softbot.global.exception.BotException;
import com.gitlink.softbot.service.market.IMarketService;
import com.gitlink.softbot.service.user.impl.UserService;
import com.gitlink.softbot.utils.GitLinkApi;
import com.gitlink.softbot.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MarketService implements IMarketService {


    @Resource
    MarketBotMapper marketBotMapper;

    @Resource
    BotMapper botMapper;

    @Resource
    BotCategoryMapper botCategoryMapper;

    @Resource
    RegisterMapper registerMapper;

    @Resource
    BotLimitMapper botLimitMapper;

    @Autowired
    GitLinkApi api;

    @Autowired
    UserService userService;

    @Override
    public MarketBotPagesVO getMarketBotByNameLike(String keyword,Integer pageIndex,Integer pageSize){
        log.info("执行Bot通过上市名称模糊查询，keyword:{},pageIndex:{},pageSize:{}",keyword,pageIndex,pageSize);
        LambdaQueryWrapper<com.gitlink.softbot.entity.db.MarketBot> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.like(com.baomidou.mybatisplus.core.toolkit.StringUtils.isNotBlank(keyword), com.gitlink.softbot.entity.db.MarketBot::getMarketName,keyword);
        lambdaQueryWrapper.orderByDesc(com.gitlink.softbot.entity.db.MarketBot::getInstallNum);
        IPage<com.gitlink.softbot.entity.db.MarketBot>  page = marketBotMapper.selectPage(new Page<com.gitlink.softbot.entity.db.MarketBot>(pageIndex,pageSize),lambdaQueryWrapper);
        MarketBotPagesVO marketBotPagesVO = new MarketBotPagesVO();
        marketBotPagesVO.setBotList(page.getRecords());
        marketBotPagesVO.setTotal(page.getTotal());
        marketBotPagesVO.setPageSize(pageIndex);
        marketBotPagesVO.setPageSize(pageSize);
        return marketBotPagesVO;
    }
//    @Override
//    public MarketBotPagesVO getMarketBotByNameLike(String keyword, int pageIndex, int pageSize) throws BotException{
//        List<MarketBot> botList = new ArrayList<>();
//        log.info("执行Bot通过上市名称模糊查询，keyword:{},pageIndex:{},pageSize:{}",keyword,pageIndex,pageSize);
//        long total=0;
//        if (StringUtils.isBlank(keyword)){
//            throw new BotException("keyword is null!");
//        }
//        try{
//            //1.创建请求
//            SearchRequest searchRequest = new SearchRequest("marketbot");
//
//            //分页
//            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
//            searchSourceBuilder.from((pageIndex-1)*pageSize+1).size(pageSize);
//
////            QueryBuilder queryBuilder = QueryBuilders.boolQuery();
//            MultiMatchQueryBuilder matchQueryBuilder = QueryBuilders.multiMatchQuery(keyword, "marketName","marketDesc");
//
//            //多个条件组合
//            searchSourceBuilder.query(matchQueryBuilder);
//
//            //设置排序
//            searchSourceBuilder.sort(new FieldSortBuilder("installNum").order(SortOrder.DESC));
//            searchSourceBuilder.sort(new FieldSortBuilder("marketTime").order(SortOrder.DESC));
//
//            //设置一个可选的超时时间，控制检索时间
//            searchSourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
//            searchRequest.source(searchSourceBuilder);
//
//            //2.执行检索请求，获取检索结果
//            SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
//
//
//            //3.解析检索结果
//            SearchHits hits = searchResponse.getHits();
//            total = hits.getTotalHits().value;
//            SearchHit[] searchHits = hits.getHits();
//            for (SearchHit hit : searchHits){
//                String index = hit.getIndex();//索引
//                String id = hit.getId();//文档id
//                log.info("根据marketName查询，索引为：{},ID为：{}",index,id);
//
//                String sourceAsString = hit.getSourceAsString();
//                MarketBot marketBot = JSON.parseObject(sourceAsString,MarketBot.class);
//                botList.add(marketBot);
//            }
//
//        }catch (Exception e){
//            throw new BotException(e.getMessage());
//        }
//        MarketBotPagesVO marketBotPagesVO = new MarketBotPagesVO();
//        marketBotPagesVO.setBotList(botList);
//        marketBotPagesVO.setTotal(total);
//        marketBotPagesVO.setPageIndex(pageIndex);
//        marketBotPagesVO.setPageSize(pageSize);
//        return marketBotPagesVO;
//    }

//    @Override
//    public MarketBotPagesVO getMarketBotByCategory(String func, int pageIndex, int pageSize) throws BotException{
//
//        log.info("执行Bot根据种类检索操作，func:{},pageIndex:{},pageSize{}",func,pageIndex,pageSize);
//        BoolQueryBuilder bdq = QueryBuilders.boolQuery();
//        List<MarketBot> botList = new ArrayList<>();
//        long total=0;
//
//        try {
//            //1.创建检索请求
//            SearchRequest searchRequest = new SearchRequest("marketbot");
//
//            //分页
//            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
//            searchSourceBuilder.from((pageIndex-1)*pageSize+1).size(pageSize);
//
//            QueryBuilder queryBuilder = QueryBuilders.boolQuery();
//            //查询条件一：
//            //如果功能为空，则表示首页的内容
//            if (!StringUtils.isBlank(func)){
//                MatchPhraseQueryBuilder mpql = QueryBuilders.matchPhraseQuery("firstFunc", func);
//                ((BoolQueryBuilder) queryBuilder).must(mpql);
//            }
//            //多个条件组合
//            searchSourceBuilder.query(queryBuilder);
//
//            //设置排序
//            searchSourceBuilder.sort(new FieldSortBuilder("installNum").order(SortOrder.DESC));
//            searchSourceBuilder.sort(new FieldSortBuilder("marketTime").order(SortOrder.DESC));
//
//            //设置一个可选的超时时间，控制检索时间
//            searchSourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
//            searchRequest.source(searchSourceBuilder);
//
//            //2.执行检索请求，获取检索结果
//            SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
//
//
//            //3.解析检索结果
//            SearchHits hits = searchResponse.getHits();
//            total = hits.getTotalHits().value;
//            SearchHit[] searchHits = hits.getHits();
//            for (SearchHit hit : searchHits){
//                String index = hit.getIndex();//索引
//                String id = hit.getId();//文档id
//                log.info("根据func查询,索引为：{},ID为：{}",index,id);
//
//                String sourceAsString = hit.getSourceAsString();
//                MarketBot marketBot = JSON.parseObject(sourceAsString,MarketBot.class);
//                botList.add(marketBot);
//            }
//        }catch (IOException ioException){
//            throw new BotException(ioException.getMessage());
//        }
//        MarketBotPagesVO marketBotPagesVO = new MarketBotPagesVO();
//        marketBotPagesVO.setBotList(botList);
//        marketBotPagesVO.setTotal(total);
//        marketBotPagesVO.setPageIndex(pageIndex);
//        marketBotPagesVO.setPageSize(pageSize);
//        return marketBotPagesVO;
//    }


    @Override
    public MarketBotPagesVO getMarketBotByCategory(String func, int pageIndex, int pageSize) throws BotException {
        log.info("执行Bot通过功能模糊查询，func:{},pageIndex:{},pageSize:{}",func,pageIndex,pageSize);
        LambdaQueryWrapper<com.gitlink.softbot.entity.db.MarketBot> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.like(com.baomidou.mybatisplus.core.toolkit.StringUtils.isNotBlank(func), com.gitlink.softbot.entity.db.MarketBot::getFirstFunc,func);
        lambdaQueryWrapper.or().like(com.baomidou.mybatisplus.core.toolkit.StringUtils.isNotBlank(func), com.gitlink.softbot.entity.db.MarketBot::getSecondFunc,func);
        lambdaQueryWrapper.orderByDesc(com.gitlink.softbot.entity.db.MarketBot::getInstallNum);
        IPage<com.gitlink.softbot.entity.db.MarketBot>  page = marketBotMapper.selectPage(new Page<com.gitlink.softbot.entity.db.MarketBot>(pageIndex,pageSize),lambdaQueryWrapper);
        MarketBotPagesVO marketBotPagesVO = new MarketBotPagesVO();
        marketBotPagesVO.setBotList(page.getRecords());
        marketBotPagesVO.setTotal(page.getTotal());
        marketBotPagesVO.setPageSize(pageIndex);
        marketBotPagesVO.setPageSize(pageSize);
        return marketBotPagesVO;

    }

    /**
     * 主页获取bot
     * @param userId
     * @param botId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public GetBotDetailResponse getBotDetail(Integer userId, Integer botId) {
        Bot bot = botMapper.selectOne(Wrappers.<Bot>lambdaQuery()
                .eq(Bot::getId,botId));
        if(bot == null) {
            return new GetBotDetailResponse();
        }
        MarketBot marketBot = marketBotMapper.selectOne(Wrappers.<MarketBot>lambdaQuery()
                .eq(MarketBot::getBotId,botId));
        List<BotLimitEvent> botLimitEvents = botLimitMapper.selectList(new QueryWrapper<BotLimitEvent>()
                .eq("bot_id",botId));
        RegisterBot registerBot = registerMapper.selectOne(Wrappers.<RegisterBot>lambdaQuery()
                .eq(RegisterBot::getBotId,botId));
        GetBotDetailResponse getBotDetailResponse = new GetBotDetailResponse();
        BeanUtils.copyProperties(bot,getBotDetailResponse);
        if(!Objects.isNull(marketBot)){
            BeanUtils.copyProperties(marketBot,getBotDetailResponse);
        }
        BeanUtils.copyProperties(registerBot,getBotDetailResponse);
        // 设置开发者名称
        getBotDetailResponse.setDeveloperName(api.getUserNameByUid(registerBot.getDeveloperId().toString()));
        getBotDetailResponse.setDeveloperLogin(registerBot.getDeveloperLogin());
        getBotDetailResponse.setLimitAndEvents(userService.getLimitFromLimitEvents(botLimitEvents));
        return getBotDetailResponse;
    }

    @Override
    public GetAllBotCategory getAllBotCategory() {
        GetAllBotCategory getAllBotCategory = new GetAllBotCategory();
       List<BotCategory>  botCategories = botCategoryMapper.selectList(null);
        if (!ObjectUtils.isEmpty(botCategories)){
            List<String> categories = botCategories.stream().map(BotCategory::getCategory).collect(Collectors.toList());
            getAllBotCategory.setCategoryList(categories);
        }else {
            getAllBotCategory.setCategoryList(Collections.EMPTY_LIST);
        }
        return getAllBotCategory;
    }

    /**
     * 根据功能和名称模糊查询
     * @param name
     * @param fun
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public MarketBotPagesVO1 getContentsLikeNameAndFunc(String name, String fun, Integer pageNo, Integer pageSize) {
        LambdaQueryWrapper<MarketBot> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        if(StringUtils.isNotBlank(fun)){
            lambdaQueryWrapper.and(wrapper->{
                wrapper.eq(StringUtils.isNotBlank(fun),MarketBot::getFirstFunc,fun)
                        .or().eq(StringUtils.isNotBlank(fun),MarketBot::getSecondFunc,fun);
            });
        }
        if(StringUtils.isNotBlank(name)){
            lambdaQueryWrapper.like(StringUtils.isNotBlank(name),MarketBot::getMarketName,name);
        }
        lambdaQueryWrapper.orderByDesc(MarketBot::getInstallNum);

        IPage<MarketBot> page = marketBotMapper.selectPage(new Page<MarketBot>(pageNo,pageSize),lambdaQueryWrapper);
        MarketBotPagesVO1 marketBotPagesVO1 = new MarketBotPagesVO1();
        List<MarketBotVO1> marketBotVO1s = new ArrayList<>();
        page.getRecords().forEach(MarketBot ->{
            Integer botId = MarketBot.getBotId();
            RegisterBot registerBot =  registerMapper.selectOne(Wrappers.<RegisterBot>lambdaQuery()
                    .eq(RegisterBot::getBotId,botId));
            MarketBotVO1 marketBotVO1 = new MarketBotVO1();
            BeanUtils.copyProperties(MarketBot,marketBotVO1);
            BeanUtils.copyProperties(registerBot,marketBotVO1);
            marketBotVO1.setDeveloperName(api.getUserNameByUid(registerBot.getDeveloperId().toString()));
            marketBotVO1s.add(marketBotVO1);
        });
        marketBotPagesVO1.setBotList(marketBotVO1s);
        marketBotPagesVO1.setTotal(page.getTotal());
        marketBotPagesVO1.setPageIndex(pageNo);
        marketBotPagesVO1.setPageSize(pageSize);
        return marketBotPagesVO1;
    }
}
