package com.gitlink.softbot.service.user.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.gitlink.softbot.dao.db.InstallMapper;
import com.gitlink.softbot.entity.db.BotLimitEvent;
import com.gitlink.softbot.entity.db.InstallBot;
import com.gitlink.softbot.global.vo.Response;
import com.gitlink.softbot.utils.GitLinkApi;
import com.gitlink.softbot.utils.response.AddWebhookResponse;
import com.gitlink.softbot.vo.Webhook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.Recover;
import org.springframework.retry.annotation.Retryable;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

@Component
@Slf4j
public class AsyncAddWebhookService {

    @Resource
    @Lazy
    private UserService userService;

    @Autowired
    private GitLinkApi api;

    @Resource
    private InstallMapper installMapper;

    @Async("addWebhookTaskExecutor")
    @Retryable(value = RuntimeException.class, maxAttempts = 3, backoff = @Backoff(delay = 2000,multiplier = 1.5))
    public void asyncAddWebhookAndRetry(Integer currentUserId, List<BotLimitEvent> botLimitEvents, InstallBot installBot)  throws RuntimeException, JsonProcessingException {
        Webhook webhook = userService.getWebhook(botLimitEvents, installBot.getInstallerId());

        //先获取webhookId构造webhook
        Object[] objects = new Object[]{installBot.getRepoOwner(), installBot.getStoreRepo()};
        Response response = api.addWebhook(currentUserId, objects, webhook);
        JSONObject object =  JSONObject.parseObject(response.getData().toString());
        if (response.getCode().equals("error") || (object.get("status")!=null&&!object.get("status").equals(0))) {
            installBot.setWebhookId(-1); //添加webhook失败
            object.put("webhook_operation_msg", "create webhook fail");
            installBot.setWebhookResponseMsg(object.toJSONString());
            installMapper.updateById(installBot);
            throw new RuntimeException("添加webhook异常");
        }
        if (response.getCode().equals("ok")) {
            //回写WebhookId
            AddWebhookResponse addWebhookResponse  = JSON.parseObject(response.getData().toString(),AddWebhookResponse.class);
            installBot.setWebhookId(addWebhookResponse.getId());
            object.put("webhook_operation_msg", "create webhook success");
            installBot.setWebhookResponseMsg(object.toJSONString());
            int updateNum = installMapper.updateById(installBot);
            //bot安装数据更新webhook_id字段失败时，删除仓库webhook
            if (updateNum == 0) {
                api.deleteWebhook(currentUserId, objects, addWebhookResponse.getId());
            }
        }

    }

    /**
     * 最终重试失败处理
     */
    @Recover
    public void recover(RuntimeException e){
        log.info("调用添加webhook请求，重试3次后依旧失败");
    }
}
