package com.gitlink.softbot.service.user.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.gitlink.softbot.dao.db.InstallMapper;
import com.gitlink.softbot.entity.db.BotLimitEvent;
import com.gitlink.softbot.entity.db.InstallBot;
import com.gitlink.softbot.global.vo.Response;
import com.gitlink.softbot.utils.GitLinkApi;
import com.gitlink.softbot.vo.Webhook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.Recover;
import org.springframework.retry.annotation.Retryable;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

@Component
@Slf4j
public class AsyncDeleteWebhookService {

    @Resource
    private UserService userService;

    @Autowired
    private GitLinkApi api;

    @Resource
    private InstallMapper installMapper;

    @Async("deleteWebhookTaskExecutor")
    @Retryable(value = RuntimeException.class, maxAttempts = 3, backoff = @Backoff(delay = 2000,multiplier = 1.5))
    public void asyncDeleteWebhookAndRetry(Integer currentUserId, InstallBot installBot)  throws RuntimeException {

        Object[] objects = new Object[]{installBot.getRepoOwner(), installBot.getStoreRepo()};
        Response response = api.deleteWebhook(currentUserId, objects, installBot.getWebhookId());
        JSONObject object =  JSONObject.parseObject(response.getData().toString());
        if (response.getCode().equals("error") || (object.get("status")!=null&&!object.get("status").equals(0)&&!object.get("status").equals(404))) {
            installBot.setWebhookId(-3); //删除webhook失败
            object.put("webhook_operation_msg", "delete webhook fail");
            installBot.setWebhookResponseMsg(object.toJSONString());
            installMapper.updateById(installBot);
            throw new RuntimeException("删除webhook异常");
        }
        if (response.getCode().equals("ok")) {
            object.put("webhook_operation_msg", "delete webhook success");
            installBot.setWebhookResponseMsg(object.toJSONString());
            installMapper.updateById(installBot);
        }

    }

    /**
     * 最终重试失败处理
     */
    @Recover
    public void recover(RuntimeException e){
        log.info("调用删除webhook请求，重试3次后依旧失败");
    }
}
