package com.gitlink.softbot.service.user.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.gitlink.softbot.dao.db.*;
import com.gitlink.softbot.entity.db.*;
import com.gitlink.softbot.global.exception.BotException;
import com.gitlink.softbot.global.vo.Response;
import com.gitlink.softbot.service.market.IMarketService;
import com.gitlink.softbot.service.user.AbstractUserBot;
import com.gitlink.softbot.service.user.IUserService;
import com.gitlink.softbot.utils.AuthOperate;
import com.gitlink.softbot.utils.GitLinkApi;
import com.gitlink.softbot.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.DefaultTransactionDefinition;
import org.springframework.web.util.UriComponentsBuilder;

import java.util.*;
import java.util.stream.Collectors;


@Service
@Slf4j
public class UserService extends AbstractUserBot implements IUserService {

    @Autowired
    BotMapper botMapper;
    @Autowired
    InstallMapper installMapper;
    @Autowired
    RegisterMapper registerMapper;
    @Autowired
    BotLimitMapper botLimitMapper;
    @Autowired
    MarketBotMapper marketBotMapper;
    @Autowired
    IMarketService marketService;
    @Autowired
    TransferBotMapper transferBotMapper;


    @Autowired
    private PlatformTransactionManager transactionManager;

    @Autowired
    GitLinkApi api;

    @Autowired
    private AsyncAddWebhookService asyncAddWebhookService;

    @Autowired
    private AsyncUpdateWebhookService asyncUpdateWebhookService;

    @Autowired
    private AsyncDeleteWebhookService asyncDeleteWebhookService;

    /**
     * 获取个人注册bot信息
     */
    @Override
    public GetRegisterBotResponse getRegisterBot(Integer userId,Integer botId) throws BotException{

        RegisterBot registerBot = registerMapper.selectOne(Wrappers.<RegisterBot>lambdaQuery()
                .eq(RegisterBot::getBotId,botId)
                .eq(RegisterBot::getDeveloperId,userId));
        log.info("Register:{}",registerBot);
        if (registerBot==null){
            throw new BotException("该Bot没有注册！");
        }
        Bot bot = botMapper.selectById(botId);
        if(bot.getPrivateKey()==null||bot.getPrivateKey().equals("")){
            Response response = api.activeBotAuth(registerBot.getDeveloperId(), bot.getId(), AuthOperate.AUTH_ACTIVE);
            checkException(response);
            bot = botMapper.selectById(botId);
        }
        List<BotLimitEvent> botLimitEvents = botLimitMapper.selectList(Wrappers.<BotLimitEvent>lambdaQuery()
                .eq(BotLimitEvent::getBotId,botId));
        LimitVO limitVO = getLimitFromLimitEvents(botLimitEvents);
        GetRegisterBotResponse getRegisterBotResponse = new GetRegisterBotResponse();
        BotOutputVO botOutputVO = new BotOutputVO();
        //增加用户登录名
        botOutputVO.setLogin(registerBot.getDeveloperLogin());
        BeanUtils.copyProperties(bot,botOutputVO);
        getRegisterBotResponse.setLogin(registerBot.getDeveloperLogin());
        botOutputVO.setLimitAndEvents(limitVO);

        //查询是否传让
        List<TransferBot> transferBots = transferBotMapper.selectList(Wrappers.<TransferBot>
                lambdaQuery().eq(TransferBot::getBotId,botId)
                .eq(TransferBot::getTransferFromId,userId)
                .orderByDesc(TransferBot::getId));
        if (transferBots!=null&&transferBots.size()>0){
            TransferBot transferBot = transferBots.get(0);
            botOutputVO.setIsTransfer(1);
            botOutputVO.setIsTransferSuccess(transferBot.getIsSuccess());
            botOutputVO.setTransferToId(transferBot.getTransferToId());
            botOutputVO.setTransferToLogin(transferBot.getToLogin());
            botOutputVO.setTransferToName(api.getUserNameByUid(transferBot.getTransferToId().toString()));
        }else {
           botOutputVO.setIsTransfer(0);
           botOutputVO.setIsTransferSuccess(3);
        }
        getRegisterBotResponse.setBotOutputVO(botOutputVO);
        getRegisterBotResponse.setUserId(userId);
        return getRegisterBotResponse;
    }

    @Override
//    @Transactional(rollbackFor = Exception.class)
    public BotOutputVO registerBot(BotInputVO botVO) throws BotException {
        log.info("开始注册Bot{}", JSON.toJSONString(botVO));
        //校验必填信息信息
        checkInfo(botVO);
        TransactionStatus transactionStatus = null;
        DefaultTransactionDefinition transactionDefinition;
        Bot insertBot;
        try {
            // 有事务，则使用当前事务；否则开启新事务
            transactionDefinition = new DefaultTransactionDefinition();
            transactionDefinition.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
            transactionStatus = transactionManager.getTransaction(transactionDefinition);
            //插入Bot表
            botMapper.insert(shiftObject(botVO));
            insertBot = botMapper.selectOne(new QueryWrapper<Bot>().eq("bot_name",botVO.getBotName()));

            //插入权限表
            List<BotLimitEvent> botLimitEvent = shiftBotToLimit(botVO,insertBot);
            botLimitEvent.forEach(botLimitEvent1 -> botLimitMapper.insert(botLimitEvent1));

            //插入注册表
            RegisterBot registerBot = shiftBotToRegisterBot(botVO,insertBot);
            registerMapper.insert(registerBot);
            log.info("注册Bot结束{}",JSON.toJSONString(shiftBotToBotOutputVo(botVO,insertBot)));

            transactionManager.commit(transactionStatus);
        } catch (Exception e){
            if (transactionStatus != null) {
                transactionManager.rollback(transactionStatus);
            }
            throw new BotException("Bot注册失败：" + e);
        }

        //激活Bot
        if(insertBot!=null){
            Response response = api.activeBotAuth(botVO.getUserId(), insertBot.getId(), AuthOperate.AUTH_ACTIVE);
            //校验异常
            checkException(response);
            log.info("注册激活信息：{}",response);
        }
        return shiftBotToBotOutputVo(botVO,insertBot);
    }

    private void checkException(Response response) throws BotException{
        JSONObject object =  JSONObject.parseObject(response.getData().toString());
        if (object.get("status")!=null&&!object.get("status").equals(0)){
            log.info("远程接口调用异常:{}",response.getData().toString());
            throw new BotException("远程接口调用异常");
        }
    }

    private void checkException(Response response, Integer rightCode) throws BotException{
        JSONObject object =  JSONObject.parseObject(response.getData().toString());
        if (object.get("status")!=null&&!object.get("status").equals(0)&&!object.get("status").equals(rightCode)){
            log.info("远程接口调用异常:{}",response.getData().toString());
            throw new BotException("远程接口调用异常");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BotOutputVO updateBot(BotInputVO botInputVO) throws BotException {
        log.info("开始配置Bot{}",JSON.toJSONString(botInputVO));
        // 检查 修改名称是够有重复
        checkBotName(botInputVO.getBotId(),botInputVO.getBotName());
        //校验是否将权限改为私有
        if (botInputVO.getIsPublic()==0){
            //判断是数据库中有没有安装的Bot，有的话返回异常
            Bot bot = botMapper.selectById(botInputVO.getBotId());
            if(bot.getIsPublic()==1){
                List<InstallBot> installBots = installMapper.selectList(new QueryWrapper<InstallBot>()
                        .eq("bot_id",bot.getId())
                        .ne("installer_id",bot.getOwnerId()));
                if (installBots!=null && installBots.size()>0){
                    throw new BotException("已有用户安装该Bot，不能将权限设置为私有");
                }
            }
        }
        //先更新Bot
        Bot updateBot = shiftObject(botInputVO);
        Integer id = botInputVO.getBotId();
        updateBot.setId(id);
        try {
        MarketBot marketBot = marketBotMapper.selectOne(new QueryWrapper<MarketBot>().eq("bot_id",id));
       if (marketBot!=null){
           marketBot.setMarketIntro(updateBot.getBotDes());
           marketBot.setLogo(updateBot.getLogo());
           marketBot.setMarketName(updateBot.getBotName());
           marketBotMapper.updateById(marketBot);
       }
            botMapper.updateById(updateBot);
            //同步更新回调地址
            Response response = api.updateCallbackUrl(botInputVO.getUserId(), updateBot.getId());
            //校验异常
            checkException(response);
        }catch (DuplicateKeyException e){
            throw new BotException("Bot更新失败！");
        }

        boolean flag = false;
        List<BotLimitEvent>  oldBotLimitEvents = botLimitMapper.selectList(new QueryWrapper<BotLimitEvent>()
                .eq("bot_id",id));

        //判断权限是否发生变化
        LimitVO limitVO = botInputVO.getLimitAndEvents();
        List<Limit> limits = limitVOToLimit(limitVO);
        flag = judgeLimitIsChange(oldBotLimitEvents,limits);
        List<BotLimitEvent> insetBotLimitEvents = shiftBotToLimit(botInputVO,updateBot);


        //有变化需要更新
        if (flag){//权限发生变化则通知安装者
            //TODO    通知权限发生变化
            //更新权限
            botLimitMapper.delete(new QueryWrapper<BotLimitEvent>()
                    .eq("bot_id",id));
            insetBotLimitEvents.forEach(insetBotLimitEvent->{
                botLimitMapper.insert(insetBotLimitEvent);
            });
        }
        //当Webhook发生变化，对所有安装此Bot的仓库进行更新webhook
        //更新Webhook
        List<InstallBot> installBotList = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                .eq(InstallBot::getBotId,botInputVO.getBotId()));
        try {
            if (!Objects.isNull(installBotList)) {
                for (InstallBot installBot : installBotList) {
                    //异步更新webhook
                    asyncUpdateWebhookService.asyncUpdateWebhookAndRetry(installBot.getInstallerId(), insetBotLimitEvents, installBot);

                }
            }
        }catch (Exception e){
            throw new BotException("Webhook更新失败！");
        }
        log.info("结束配置Bot{}",JSON.toJSONString(shiftBotToBotOutputVo(botInputVO,updateBot)));
        return shiftBotToBotOutputVo(botInputVO,updateBot);
    }

    private List<Limit> limitVOToLimit(LimitVO limitVO) throws BotException{
        log.info("权限信息：{}",limitVO);
        String eventCode = limitVO.getEventCode();
        String eventPr = limitVO.getEventPr();
        String eventIssue = limitVO.getEventIssue();
        //无权限条件下
        if (StringUtils.isBlank(eventCode) && StringUtils.isBlank(eventPr) && StringUtils.isBlank(eventIssue)){
            throw new BotException("订阅事件不能为空！");
        }
        //0：只读1：读写2:无权限
        Integer jurisDictionCode = limitVO.getJurisDictionCode();
        //0：只读1：读写2:无权限
        Integer jurisDictionPr = limitVO.getJurisDictionPr();

        Integer jurisDictionIssue = limitVO.getJurisDictionIssue();

        if ((jurisDictionCode == 2) && (jurisDictionPr == 2) && (jurisDictionIssue == 2)){
            throw new BotException("仓库访问权限不能为空！");
        }

        if (jurisDictionCode == 2 && !StringUtils.isBlank(eventCode)
        || (jurisDictionPr == 2 && !StringUtils.isBlank(eventPr))
        || (jurisDictionIssue == 2 && !StringUtils.isBlank(eventIssue))) {
            throw new BotException("权限&订阅设置错误！");
        }

        String[] codeEvents = eventCode.split(",");
        String[] prEvents = eventPr.split(",");
        String[] issueEvents = eventIssue.split(",");
        List<Integer> eventList = new ArrayList<>();
        for (String s : codeEvents){
            if (StringUtils.isBlank(s)){
                break;
            }
            eventList.add(Integer.valueOf(s));
        }
        for (String s : prEvents){
            if (StringUtils.isBlank(s)){
                break;
            }
           eventList.add(Integer.valueOf(s));
        }
        for (String s : issueEvents){
            if (StringUtils.isBlank(s)){
                break;
            }
            eventList.add(Integer.valueOf(s));
        }
        List<Limit> limitList = new ArrayList<>(eventList.size());

        eventList.forEach(event ->{
            switch (event) {
                case 0: //代码库推送
                case 1: //代码库创建
                case 2: //代码库删除
                    Limit codeLimit = new Limit();
                    codeLimit.setEvent(event);
                    codeLimit.setAuthCategory(0);
                    //代码和请求字段在一条记录中，有一个必须为无权限
                    codeLimit.setReadWriteCode(jurisDictionCode);
                    codeLimit.setReadWritePr(2);
                    codeLimit.setReadWriteIssue(2);
                    limitList.add(codeLimit);
                    break;
                case 3: //合并请求被打开、被关闭、被重新打开或被编辑
                case 4: //合并请求被分配或取消分配
                case 6: //合并请求评论被创建、编辑或删除
                    Limit prLimit = new Limit();
                    prLimit.setEvent(event);
                    prLimit.setAuthCategory(1);
                    prLimit.setReadWritePr(jurisDictionPr);
                    prLimit.setReadWriteCode(2);
                    prLimit.setReadWriteIssue(2);
                    limitList.add(prLimit);
                    break;
                case 7: //疑修已打开、已关闭、已重新打开或编辑
                case 8: //疑修已被指派或取消指派
                case 9: //疑修标记被更新或清除
                case 10: //疑修评论被创建、编辑或删除
                    Limit issueLimit = new Limit();
                    issueLimit.setEvent(event);
                    issueLimit.setAuthCategory(3);
                    issueLimit.setReadWritePr(2);
                    issueLimit.setReadWriteCode(2);
                    issueLimit.setReadWriteIssue(jurisDictionIssue);
                    limitList.add(issueLimit);
                    break;
            }
        });
      return limitList;
    }
    @Override
    public GetBotResponse getBots(Integer userId) {
        List<RegisterBot> registerBots = registerMapper.selectList(Wrappers.<RegisterBot>lambdaQuery()
                .eq(RegisterBot::getDeveloperId,userId));
        List<BotOutputVO> botOutputVOS = new ArrayList<>();
        registerBots.forEach(registerBot -> {
            botOutputVOS.add(getBotOutputVoFromLimitAndBot(
                   botMapper.selectById(registerBot.getBotId()),botLimitMapper.selectList(new QueryWrapper<BotLimitEvent>()
                            .eq("bot_id",registerBot.getBotId())),userId,registerBot.getDeveloperLogin()
            ));
        });
        botOutputVOS.forEach(botOutputVO->{
            botOutputVO.setIsMarket(
                    Objects.isNull(marketBotMapper.selectOne(new QueryWrapper<MarketBot>().eq("bot_id",botOutputVO.getBotId())))?0:1);
        });
        GetBotResponse getBotResponse = new GetBotResponse();
        getBotResponse.setUserId(userId);
        getBotResponse.setList(botOutputVOS);
        return getBotResponse;
    }

    @Override
    public void checkBotName(Integer botId, String botName) throws BotException {
        QueryWrapper<Bot> wrapper = new QueryWrapper<>();
        wrapper.eq("bot_name",botName);
        List<Bot> botList = botMapper.selectList(wrapper);
        if (botList!=null && botList.size()>0 ){
            if(!botId.equals(botList.get(0).getId())){
                throw new BotException("该名称已经被使用！");
            }
        }
    }

    @Override
    public void checkInfo(BotInputVO botVO) throws BotException {

            if (botVO.getUserId()==null){
                throw new BotException("Bot注册失败：userId不能为空!");
            }
            if (StringUtils.isBlank(botVO.getBotName())){
                throw new BotException("Bot注册失败：名称不能为空!");
            }
            if (StringUtils.isBlank(botVO.getWebhook())){
                throw new BotException("Bot注册失败：webhook不能为空!");
            }

        QueryWrapper<Bot> wrapper = new QueryWrapper<>();
        wrapper.eq("bot_name",botVO.getBotName());
            List<Bot> botList = botMapper.selectList(wrapper);
            if (botList.size()>0){
                throw new BotException("Bot注册失败：该名称已经被使用！");
            }
    }
    public BotOutputVO getBotOutputVoFromLimitAndBot(Bot bot,List<BotLimitEvent> botLimitEvents,Integer userId,String login){
        BotOutputVO botOutputVO = new BotOutputVO();
        botOutputVO.setBotId(bot.getId());
        botOutputVO.setBotDes(bot.getBotDes());
        botOutputVO.setBotName(bot.getBotName());
        botOutputVO.setUserId(userId);
        botOutputVO.setLogin(login);
        botOutputVO.setLogo(bot.getLogo());
        botOutputVO.setWebUrl(bot.getWebUrl());
        botOutputVO.setWebhook(bot.getWebhook());
        botOutputVO.setClientId(bot.getClientId());
        botOutputVO.setClientSecret(bot.getClientSecret());
        botOutputVO.setLimitAndEvents(getLimitFromLimitEvents(botLimitEvents));
        botOutputVO.setIsPublic(bot.getIsPublic());
        return botOutputVO;
    }
    public LimitVO getLimitFromLimitEvents(List<BotLimitEvent> botLimitEvents){
        LimitVO limitVO = new LimitVO();
        //无权限情况下
        if (botLimitEvents.size()==1&&botLimitEvents.get(0).getEvent()==5){
            limitVO.setEventPr("");
            limitVO.setEventCode("");
            limitVO.setEventIssue("");
            limitVO.setJurisDictionCode(2);
            limitVO.setJurisDictionPr(2);
            limitVO.setJurisDictionIssue(2);
        }
        StringJoiner codeStr = new StringJoiner(",");
        StringJoiner prStr = new StringJoiner(",");
        StringJoiner issueStr = new StringJoiner(",");

        botLimitEvents.forEach(botLimitEvent -> {

            switch (botLimitEvent.getEvent()) {
                case 0:
                case 1:
                case 2:
                    codeStr.add(botLimitEvent.getEvent().toString());
                    limitVO.setJurisDictionCode(botLimitEvent.getReadWriteCode());break;
                case 3:
                case 4:
                case 6:
                    prStr.add(botLimitEvent.getEvent().toString());
                    limitVO.setJurisDictionPr(botLimitEvent.getReadWritePr());break;
                case 7:
                case 8:
                case 9:
                case 10:
                    issueStr.add(botLimitEvent.getEvent().toString());
                    limitVO.setJurisDictionIssue(botLimitEvent.getReadWriteIssue());break;
            }
        });

        if (codeStr.length()==0) {
            limitVO.setJurisDictionCode(2);
        }
        if (prStr.length()==0) {
            limitVO.setJurisDictionPr(2);
        }
        if (issueStr.length() == 0) {
            limitVO.setJurisDictionIssue(2);
        }

        limitVO.setEventPr(prStr.toString());
        limitVO.setEventCode(codeStr.toString());
        limitVO.setEventIssue(issueStr.toString());

        log.info("limitVO:{}",limitVO);
        return limitVO;
    }
    @Override
    public Bot shiftObject(BotInputVO botVO) {
        Bot bot = new Bot();
        bot.setBotDes(botVO.getBotDes());
        bot.setBotName(botVO.getBotName());
        bot.setIsPublic(botVO.getIsPublic());
        bot.setWebhook(botVO.getWebhook());
        bot.setWebUrl(botVO.getBotUrl());
        bot.setLogo(botVO.getLogo());
        bot.setState(0);
        bot.setOauthCallbackUrl(botVO.getOauthCallbackUrl());
        return bot;
    }
    public List<BotLimitEvent> shiftBotToLimit(BotInputVO botInputVO,Bot bot) throws BotException{

        LimitVO limitVO = botInputVO.getLimitAndEvents();
        List<Limit>  limitList = limitVOToLimit(limitVO);
        List<BotLimitEvent> botLimitEvents = new ArrayList<>();
        limitList.forEach(limit -> {
            BotLimitEvent botLimitEvent = new BotLimitEvent();
            botLimitEvent.setBotId(bot.getId());
            botLimitEvent.setAuthCategory(limit.getAuthCategory());
            botLimitEvent.setReadWriteCode(limit.getReadWriteCode());
            botLimitEvent.setReadWritePr(limit.getReadWritePr());
            botLimitEvent.setReadWriteIssue(limit.getReadWriteIssue());
            botLimitEvent.setEvent(limit.getEvent());
            botLimitEvents.add(botLimitEvent);
        });
        return botLimitEvents;
    }
    public RegisterBot shiftBotToRegisterBot(BotInputVO botInputVO,Bot bot){
        RegisterBot registerBot = new RegisterBot();
        registerBot.setBotId(bot.getId());
        registerBot.setDeveloperId(botInputVO.getUserId());
        registerBot.setDeveloperLogin(botInputVO.getLogin());
        return registerBot;
    }
    public BotOutputVO shiftBotToBotOutputVo(BotInputVO botInputVO,Bot bot){
        BotOutputVO botOutputVO = new BotOutputVO();
        botOutputVO.setBotId(bot.getId());
        botOutputVO.setBotDes(bot.getBotDes());
        botOutputVO.setBotName(bot.getBotName());
        botOutputVO.setClientId(bot.getClientId());
        botOutputVO.setClientSecret(bot.getClientSecret());
        botOutputVO.setIsPublic(bot.getIsPublic());
        botOutputVO.setWebhook(bot.getWebhook());
        botOutputVO.setUserId(botInputVO.getUserId());
        botOutputVO.setWebUrl(bot.getWebUrl());
        botOutputVO.setLimitAndEvents(botInputVO.getLimitAndEvents());
        botOutputVO.setLogo(bot.getLogo());
        botOutputVO.setOauthCallbackUrl(bot.getOauthCallbackUrl());
        return botOutputVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteBot(DeleteBotRequest deleteBotRequest) throws BotException{
        log.info("删除Bot:{}",JSON.toJSONString(deleteBotRequest));
        //删除Bot涉及Bot表、Limit表、register表、install表、es中数据,Market中数据、删除transfer中数据
        //1.删除Bot中数据
        RegisterBot registerBot = registerMapper.selectOne(Wrappers.<RegisterBot>
                lambdaQuery().eq(RegisterBot::getBotId,deleteBotRequest.getBotId()));
        if (registerBot==null){
            throw new BotException("要删除的Bot不存在！");
        }
        if (!registerBot.getDeveloperId().equals(deleteBotRequest.getUserId())){
            throw new BotException("不能删除不属于自己的Bot！");
        }

        //查询所有安装要删除bot的安装信息
        List<InstallBot> installBots = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                .eq(InstallBot::getBotId,deleteBotRequest.getBotId()));
        if (installBots!=null&&installBots.size()>0){

            //批量删除webhook
            for (InstallBot installBot : installBots) {

                //调用平台api，删除所有相关webhook
                asyncDeleteWebhookService.asyncDeleteWebhookAndRetry(installBot.getInstallerId(), installBot);

            }

        }

        int flag1 = botMapper.deleteById(deleteBotRequest.getBotId());
        //2.删除limit表中数据
        Map<String,Object> map = new HashMap<>();
        map.put("bot_id",deleteBotRequest.getBotId());
        int flag2 =  botLimitMapper.deleteByMap(map);
        //删除register表中数据
        int flag3 = registerMapper.deleteByMap(map);
        //删除install表中数据
        int flag4 = installMapper.deleteByMap(map);
        //删除Market表中数据
        int flag5 = marketBotMapper.deleteByMap(map);
//        //删除es中数据
//        marketService.deleteMarketBot(String.valueOf(deleteBotRequest.getBotId()));
        //删除正在转让的bot
        transferBotMapper.deleteByMap(map);
        //TODO 告知平台该bot不生效
        //TODO 通知用户
        log.info("删除Bot成功:{}",JSON.toJSONString(deleteBotRequest));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void botToMarket(BotToMarketRequest botToMarketRequest) throws BotException{
        log.info("Bot上市{}",botToMarketRequest);
        //先校验是否公开，不是公开，则判断
        Bot bot = botMapper.selectById(botToMarketRequest.getBotId());
        if (bot==null){
            throw new BotException("该Bot不存在!");
        }
        if (bot!=null&&bot.getIsPublic()!=1){
            throw new BotException("该Bot未公开！");
        }
        //判断Bot上市名称是否重复
        checkBotName(botToMarketRequest.getBotId(),botToMarketRequest.getMarketName());
        try {
            List<MarketBot> marketBotList = marketBotMapper.selectList(new QueryWrapper<MarketBot>()
                    .eq("market_name", botToMarketRequest.getMarketName()));
            if (marketBotList.size() > 0) {
                throw new BotException("此名称已被占用！");
            }
            //1.插入market_bot表
            MarketBot marketBot = getDBMarketBot(botToMarketRequest);
            //更新市场bot的安装次数，bot在上市前，可能通过详情页面被安装过
            marketBot.setInstallNum(bot.getInstallNum());

            //更新bot表中详细信息
            bot.setBotDes(marketBot.getMarketIntro());
            bot.setBotName(marketBot.getMarketName());
            bot.setLogo(marketBot.getLogo());
            botMapper.updateById(bot);
            marketBotMapper.insert(marketBot);
//            //2.es添加数据
//            com.gitlink.softbot.entity.es.MarketBot marketBot1 = getESMarketBot(botToMarketRequest);
//            marketService.saveMarketBot(marketBot1);
        }catch (DuplicateKeyException e){
            throw new BotException("上市失败！");
        }
        log.info("Bot上市成功！{}",botToMarketRequest);
    }

    public MarketBot getDBMarketBot(BotToMarketRequest botToMarketRequest){
        MarketBot marketBot = new MarketBot();
        marketBot.setMarketDesc(botToMarketRequest.getMarketDesc());
        marketBot.setBotId(botToMarketRequest.getBotId());
        marketBot.setMarketName(botToMarketRequest.getMarketName());
        marketBot.setMarketTime(new Date());
        marketBot.setCategory(botToMarketRequest.getCategory());
        marketBot.setLogo(botToMarketRequest.getLogo());
        marketBot.setFirstFunc(botToMarketRequest.getFirstFunc());
        marketBot.setSecondFunc(botToMarketRequest.getSecondFunc());
        marketBot.setInstallNum(0);
        marketBot.setWebhook(botToMarketRequest.getWebhook());
        marketBot.setIsReceiveAg1(botToMarketRequest.getIsReceiveAg1());
        marketBot.setIsReceiveAg2(botToMarketRequest.getIsReceiveAg2());
        marketBot.setMarketIntro(botToMarketRequest.getMarketIntro());
        return marketBot;
    }
    public com.gitlink.softbot.entity.es.MarketBot getESMarketBot(BotToMarketRequest botToMarketRequest){
        com.gitlink.softbot.entity.es.MarketBot marketBot = new com.gitlink.softbot.entity.es.MarketBot();
        marketBot.setId(String.valueOf(botToMarketRequest.getBotId()));
        marketBot.setMarketDesc(botToMarketRequest.getMarketDesc());
        marketBot.setMarketName(botToMarketRequest.getMarketName());
        marketBot.setMarketTime(new Date());
        marketBot.setCategory(botToMarketRequest.getCategory());
        marketBot.setLogo(botToMarketRequest.getLogo());
        marketBot.setFirstFunc(botToMarketRequest.getFirstFunc());
        marketBot.setSecondFunc(botToMarketRequest.getSecondFunc());
        marketBot.setInstallNum(0);
        marketBot.setWebhook(botToMarketRequest.getWebhook());
        marketBot.setIsReceiveAg1(botToMarketRequest.getIsReceiveAg1());
        marketBot.setIsReceiveAg2(botToMarketRequest.getIsReceiveAg2());

        return marketBot;
    }
    @Override
    public GetMarketBotsResponse getMarketBots(GetMarketBotsRequest getMarketBotsRequest) {
        //先查注册表，根据注册表中所有的bot_id

        List<RegisterBot> registerBots = registerMapper.selectList(new QueryWrapper<RegisterBot>()
                .eq("developer_id",getMarketBotsRequest.getUserId()));
        GetMarketBotsResponse getMarketBotsResponse = new GetMarketBotsResponse();
        List<GetMarketBotsResponse.GetMarketBot> botList = new ArrayList<>();
        //根据bot_id，返回所有marketBot
        registerBots.forEach(registerBot -> {

            System.out.println(registerBot.getBotId());
            MarketBot marketBot = marketBotMapper.selectOne(new QueryWrapper<MarketBot>()
                    .eq("bot_id",registerBot.getBotId()));
            if (Objects.isNull(marketBot)){
                return;
            }
            botList.add(getMarketBotsResponse.getMarketBot(
                    marketBot
            ));
        });
        getMarketBotsResponse.setUserId(getMarketBotsRequest.getUserId());
        getMarketBotsResponse.setGetMarketBots(botList);
        return getMarketBotsResponse;
    }

    @Override
    public MarketBotVO getMarketBotById(Integer botId) throws BotException{
        MarketBot marketBot = marketBotMapper.selectOne(new QueryWrapper<MarketBot>()
                .eq("bot_id",botId));
        if (marketBot==null){
            throw new BotException("该Bot不存在");
        }
        return getMarketBotById(marketBot);
    }
    public MarketBotVO getMarketBotById(MarketBot marketBot){
        MarketBotVO marketBotVO = new MarketBotVO();
        marketBotVO.setBotId(marketBot.getBotId());
        marketBotVO.setMarketDesc(marketBot.getMarketDesc());
        marketBotVO.setMarketTime(marketBot.getMarketTime());
        marketBotVO.setFirstFunc(marketBot.getFirstFunc());
        marketBotVO.setSecondFunc(marketBot.getSecondFunc());
        marketBotVO.setInstallNum(marketBot.getInstallNum());
        marketBotVO.setLogo(marketBot.getLogo());
        marketBotVO.setMarketName(marketBot.getMarketName());
        marketBotVO.setCategory(marketBot.getCategory());
        marketBotVO.setMarketIntro(marketBot.getMarketIntro());
        marketBotVO.setWebhook(marketBot.getWebhook());
        return marketBotVO;
    }

    @Override
    @Transactional(rollbackFor = BotException.class, noRollbackFor = RuntimeException.class)
    public void installMarketBot(InstallMarketBotRequest installMarketBotRequest) throws BotException, JsonProcessingException {

        try {
            //1.校验有没有安装过此Bot
            List<InstallBot> installBots = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                    .eq(InstallBot::getBotId,installMarketBotRequest.getBotId())
                    .in(InstallBot::getStoreId,installMarketBotRequest.getStoreList()));
            if (installBots.size()>0){
                throw new BotException("存在仓库已经安装过此Bot！");
            }
            //1.插入到installBot表中
            List<Integer> storeList = installMarketBotRequest.getStoreList();
            List<InstallBot> installBotList = new ArrayList<>();

            //批量构造InstallBot
            getInstallBots(installBotList,installMarketBotRequest,storeList);

            //查询订阅事件
            List<BotLimitEvent> botLimitEvents = botLimitMapper.selectList(Wrappers.<BotLimitEvent>lambdaQuery()
                    .eq(BotLimitEvent::getBotId, installMarketBotRequest.getBotId()));
            assert botLimitEvents != null && botLimitEvents.size() > 0;

            //批量插入installBot表，添加webhook
            for (InstallBot installBot : installBotList) {
                //插入表
                installMapper.insert(installBot);
                asyncAddWebhookService.asyncAddWebhookAndRetry(installMarketBotRequest.getUserId(), botLimitEvents, installBot);
            }

            //同步累加安装次数
            botInstallNumAdd(installMarketBotRequest.getBotId(), installBotList.size());
            marketBotInstallNumAdd(installMarketBotRequest.getBotId(), installBotList.size());

        }catch (BotException e){
            throw new BotException("安装失败！");
        }

    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void unInstallMarketBot(UnInstallMarketBotRequest unInstallMarketBotRequest){
        // 检查该仓库安装的bot
        List<InstallBot> installBots = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                .eq(InstallBot::getStoreId,unInstallMarketBotRequest.getStoreId())
                .eq(InstallBot::getStoreRepo,unInstallMarketBotRequest.getStoreIdentifier()));
        // 如果安装了bot
        if(installBots!=null&&installBots.size()>0){
            //删除各个webhook
//            for (InstallBot installBot : installBots) {
//                Object[] objects = new Object[]{unInstallMarketBotRequest.getLogin(), installBot.getStoreRepo()};
//                Response response = api.deleteWebhook(unInstallMarketBotRequest.getUserId(), objects, installBot.getWebhookId());
//                checkException(response);
//            }
            // 删除仓库的所有安装记录
            installMapper.delete(Wrappers.<InstallBot>lambdaQuery()
                    .eq(InstallBot::getStoreId,unInstallMarketBotRequest.getStoreId())
                    .eq(InstallBot::getStoreRepo,unInstallMarketBotRequest.getStoreIdentifier()));
        }
    }

    public Webhook getWebhook(List<BotLimitEvent> limitEvents, Integer installerId){

        assert !Objects.isNull(limitEvents);
        Webhook webhook = new Webhook();
        Object[] objects = new Object[limitEvents.size()];
        Integer event;
        for (int i = 0; i < limitEvents.size(); i++) {
            event = limitEvents.get(i).getEvent();
            switch (event) {
                case 0:
                    objects[i]="push"; break;
                case 1:
                    objects[i]="create"; break;
                case 2:
                    objects[i]="delete"; break;
                case 3:
                    objects[i]="pull_request_only"; break;
                case 4:
                    objects[i]="pull_request_assign"; break;
                case 6:
                    objects[i]="pull_request_comment"; break;
                case 7:
                    objects[i]="issues_only"; break;
                case 8:
                    objects[i]="issue_assign"; break;
                case 9:
                    objects[i]="issue_label"; break;
                case 10:
                    objects[i]="issue_comment"; break;
            }
        }
        Bot bot = botMapper.selectById(limitEvents.get(0).getBotId());
        webhook.setEvents(objects);
        webhook.setActive(true);
        webhook.setContent_type("json");
        String webhookUri = UriComponentsBuilder.fromUriString(bot.getWebhook()).queryParam("installer_id", installerId).build().toString();
        webhook.setUrl(webhookUri);
        webhook.setType("softbot");
        webhook.setHttp_method("POST");
        webhook.setBranch_filter("*");
        return webhook;
    }
    public void getInstallBots(List<InstallBot> installBots,InstallMarketBotRequest installMarketBotRequest,List<Integer> storeIds){
//        InstallBot installBot = new InstallBot();
//        installBot.setBotId(installMarketBotRequest.getBotId());
//        installBot.setInstallerId(installMarketBotRequest.getUserId());
//        installBot.setStoreId(storeId);
//        installBot.setState(installMarketBotRequest.getState());
        if (storeIds!=null&&storeIds.size()>0){
            storeIds.forEach(storeId->{
                InstallBot installBot = new InstallBot();
                installBot.setBotId(installMarketBotRequest.getBotId());
                installBot.setInstallerId(installMarketBotRequest.getUserId());
                installBot.setInstallerLogin(installMarketBotRequest.getLogin());
                installBot.setStoreId(storeId);
                installBot.setStoreRepo(installMarketBotRequest.getRepoMap().get(storeId));
                installBot.setRepoOwner(installMarketBotRequest.getRepoOwnerMap().get(storeId));
                installBot.setState(installMarketBotRequest.getState());
                installBots.add(installBot);
            });
        }
    }
    public InstallBot getInstallBot(UpdateInstallBotRequest updateInstallBotRequest,Integer storeId){
        InstallBot installBot = new InstallBot();
        installBot.setBotId(updateInstallBotRequest.getBotId());
        installBot.setInstallerId(updateInstallBotRequest.getUserId());
        installBot.setStoreId(storeId);
        installBot.setInstallerLogin(updateInstallBotRequest.getLogin());
        installBot.setStoreRepo(updateInstallBotRequest.getRepoMap().get(storeId));
        installBot.setRepoOwner(updateInstallBotRequest.getRepoOwnerMap().get(storeId));
        //TODO  如果bot生效，通过平台该bot
        installBot.setState(updateInstallBotRequest.getState());
        return installBot;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, noRollbackFor = RuntimeException.class)
    public void updateInstallBot(UpdateInstallBotRequest updateInstallBotRequest) throws Exception {

        //1.先删除botId和userId对应的所有installBot
        List<InstallBot> installBotList = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                .eq(InstallBot::getBotId,updateInstallBotRequest.getBotId())
                .in(InstallBot::getStoreId,updateInstallBotRequest.getInstalledStoreList()));

        if (!Objects.isNull(installBotList)){

            for (InstallBot installBot : installBotList){
                //先删除webhook
                asyncDeleteWebhookService.asyncDeleteWebhookAndRetry(updateInstallBotRequest.getUserId(), installBot);
            }
        }

        installMapper.delete(new QueryWrapper<InstallBot>()
                .eq("bot_id", updateInstallBotRequest.getBotId())
                .in("store_id", updateInstallBotRequest.getInstalledStoreList()));
        List<Integer> storeList = updateInstallBotRequest.getStoreList();

        //2.然后重新插入所有storeId; //新增webhook
        List<BotLimitEvent> botLimitEvents = botLimitMapper.selectList(Wrappers.<BotLimitEvent>lambdaQuery()
                .eq(BotLimitEvent::getBotId,updateInstallBotRequest.getBotId()));


        for (Integer store : storeList){
            InstallBot installBot = getInstallBot(updateInstallBotRequest,store);
            installMapper.insert(installBot);
            asyncAddWebhookService.asyncAddWebhookAndRetry(updateInstallBotRequest.getUserId(), botLimitEvents, installBot);
        }

        //同步累加安装次数
        botInstallNumAdd(updateInstallBotRequest.getBotId(), storeList.size());
        marketBotInstallNumAdd(updateInstallBotRequest.getBotId(), storeList.size());

    }

    private void botInstallNumAdd(Integer botId, int installNum) {
        UpdateWrapper<Bot> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(Bot::getId, botId)
                        .setSql("install_num = install_num + " + installNum);
        botMapper.update(null, updateWrapper);
    }

    private void marketBotInstallNumAdd(Integer botId, int installNum) {
        UpdateWrapper<MarketBot> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarketBot::getBotId, botId)
                .setSql("install_num = install_num + " + installNum);
        marketBotMapper.update(null, updateWrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteInstallBot(DeleteInstallBotRequest deleteInstallBotRequest) throws BotException{
        List<InstallBot> installBots = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                .eq(InstallBot::getBotId,deleteInstallBotRequest.getBotId())
                .in(InstallBot::getStoreId, Arrays.asList(deleteInstallBotRequest.getStoreRepoIds().split(","))));
        if (installBots==null||installBots.size()<=0){
            throw new BotException("该用户没有要删除的Bot");
        }
        //TODO 删除要验证密码

        //删除install_bot
        installMapper.delete(Wrappers.<InstallBot>lambdaQuery()
                .eq(InstallBot::getBotId,deleteInstallBotRequest.getBotId())
                .in(InstallBot::getStoreId, Arrays.asList(deleteInstallBotRequest.getStoreRepoIds().split(","))));

        //删除webhook
        for(InstallBot installBot:installBots){

            asyncDeleteWebhookService.asyncDeleteWebhookAndRetry(deleteInstallBotRequest.getUserId(), installBot);

        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public GetInstallBotResponse getInstallBot(GetInstallBotRequest getInstallBotRequest) throws BotException{
        GetInstallBotResponse getInstallBotResponse = new GetInstallBotResponse();
        //1.到bot表中查看name
        Bot bot = botMapper.selectById(getInstallBotRequest.getBotId());

        String name = "";
        String logo = "";
        int isPublic;
        if (bot!=null){
            name = bot.getBotName();
            logo = bot.getLogo();
            isPublic = bot.getIsPublic();
        }else {
            throw new BotException("不存在该bot");
        }
        getInstallBotResponse.setLogo(logo);
        getInstallBotResponse.setMarketName(name);
        getInstallBotResponse.setIsPublic(isPublic);
        //2.到install_bot表中查看state
        List<InstallBot> installBots = installMapper.selectList(new QueryWrapper<InstallBot>()
                .eq("bot_id",getInstallBotRequest.getBotId())
                .in("store_id", Arrays.asList(getInstallBotRequest.getRepoIds().split(","))));
        assert installBots.size()>0;
        getInstallBotResponse.setCreateTime(installBots.get(0).getCreateTime());
        getInstallBotResponse.setState(installBots.get(0).getState());
        //3.并且把所有storeId返回
        List<Integer> storeIdList = new ArrayList<>();
        installBots.forEach(installBot -> storeIdList.add(installBot.getStoreId()));
        getInstallBotResponse.setStoreIdList(storeIdList);

        //获取注册者
        RegisterBot registerBot = registerMapper.selectOne(Wrappers.<RegisterBot>lambdaQuery()
                .eq(RegisterBot::getBotId,getInstallBotRequest.getBotId()));
        if (registerBot!=null){
            getInstallBotResponse.setRegisterId(registerBot.getDeveloperId());
            getInstallBotResponse.setRegisterLogin(registerBot.getDeveloperLogin());
            getInstallBotResponse.setRegisterName(api.getUserNameByUid(registerBot.getDeveloperId().toString()));
        }
        //获取所有权限
        List<BotLimitEvent> botLimitEvents = botLimitMapper.selectList(Wrappers.<BotLimitEvent>
                lambdaQuery().eq(BotLimitEvent::getBotId,getInstallBotRequest.getBotId()));
        LimitVO limitVO = getLimitFromLimitEvents(botLimitEvents);
        getInstallBotResponse.setLimitVO(limitVO);
        //TODO 4.远程调用，返回该用户所有storeId
        List<Integer> allStoreIdList = getAllStoreIdList(getInstallBotRequest.getUserId());
        getInstallBotResponse.setAllStoreIdList(allStoreIdList);
        return getInstallBotResponse;
    }

    public List<Integer> getAllStoreIdList(Integer userId){
        //TODO 获取用户仓库id
        List<Integer> allStoreIdList = new ArrayList<>();
        return allStoreIdList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void transferBot(TransferBotRequest transferBotRequest) throws BotException {
        //校验待转让者与bot所属关系
        Bot bot = botMapper.selectById(transferBotRequest.getBotId());
        if(Objects.isNull(bot)||!bot.getOwnerId().equals(transferBotRequest.getUserId())){
            throw new BotException("转让的bot不存在或者不属于该开发者");
        }
        TransferBot transferBot0 = transferBotMapper.selectOne(new QueryWrapper<TransferBot>()
                .eq("bot_id",transferBotRequest.getBotId())
                .eq("is_success",2));
        if(!Objects.isNull(transferBot0)){
            throw new BotException("该bot已经处于转让状态中！");
        }
        TransferBot transferBot = new TransferBot();
        transferBot.setBotId(transferBotRequest.getBotId());
        transferBot.setTransferFromId(transferBotRequest.getUserId());
        transferBot.setFromLogin(transferBotRequest.getLogin());
        //待转让状态
        transferBot.setIsSuccess(2);
        transferBot.setTransferTime(new Date());
        transferBot.setTransferToId(transferBotRequest.getTransferToUserId());
        transferBot.setToLogin(transferBotRequest.getToLogn());
//        //修改注册人信息
//        RegisterBot registerBot = registerMapper.selectOne(Wrappers.<RegisterBot>lambdaQuery()
//                .eq(RegisterBot::getBotId,transferBotRequest.getBotId()));
//        registerBot.setDeveloperId(transferBotRequest.getTransferToUserId());
//        registerMapper.updateById(registerBot);
        transferBotMapper.insert(transferBot);
    }

    @Override
    public GetTransferFromBotResponse getTransferBotState(GetTransferFromBotRequest getTransferFromBotRequest) {
        List<TransferBot> transferBots = transferBotMapper.selectList(Wrappers.<TransferBot>lambdaQuery()
                .eq(TransferBot::getBotId,getTransferFromBotRequest.getBotId())
                .eq(TransferBot::getTransferFromId,getTransferFromBotRequest.getUserId())
                .orderByDesc(TransferBot::getId));
        GetTransferFromBotResponse getTransferFromBotResponse = new GetTransferFromBotResponse();
        if(transferBots!=null && transferBots.size()>0){
            TransferBot transferBot = transferBots.get(0);
            getTransferFromBotResponse.setBotId(getTransferFromBotRequest.getBotId());
            getTransferFromBotResponse.setUserId(getTransferFromBotRequest.getUserId());
            getTransferFromBotResponse.setState(transferBot.getIsSuccess());
            getTransferFromBotResponse.setLogin(transferBot.getFromLogin());
        }
        return getTransferFromBotResponse;
    }

    @Override
    public void cancelTransferBot(CancelTransferBotRequest cancelTransferBotRequest) throws BotException{
        List<TransferBot> transferBots = transferBotMapper.selectList(Wrappers.<TransferBot>lambdaQuery()
                .eq(TransferBot::getBotId,cancelTransferBotRequest.getBotId())
                .eq(TransferBot::getTransferFromId,cancelTransferBotRequest.getUserId())
                .eq(TransferBot::getIsSuccess,2)
                .orderByDesc(TransferBot::getId));
        if (transferBots!=null&&transferBots.size()>0){
            TransferBot transferBot = transferBots.get(0);
            transferBot.setIsSuccess(0);
            transferBotMapper.updateById(transferBot);
        }else {
            throw new BotException("不可以取消！");
        }
    }

    @Override
    public GetTransferToBotResponse getTransferToBot(GetTransferToBotRequest getTransferToBotRequest) {
        List<TransferBot> transferBots = transferBotMapper.selectList(new QueryWrapper<TransferBot>()
                .eq("is_success",getTransferToBotRequest.getState())
                .eq("transfer_to_id",getTransferToBotRequest.getUserId()));
        Map<Integer,TransferBot> transferBotMap = transferBots.
                stream().
                collect(Collectors.toMap(TransferBot::getBotId,transferBot -> transferBot));
        List<Bot> botList = new ArrayList<>();
        if (!Objects.isNull(transferBots)){
            transferBots.forEach(transferBot -> {
                Bot bot = botMapper.selectOne(new QueryWrapper<Bot>()
                        .eq("id",transferBot.getBotId()));
                if (!Objects.isNull(bot)){
                    botList.add(bot);
                }
            });
        }
        GetTransferToBotResponse getTransferToBotResponse = new GetTransferToBotResponse();
        List<GetTransferToBot> getTransferToBots = new ArrayList<>();
        if (!Objects.isNull(botList)){
            botList.forEach(bot -> {
                log.info("bot==:{}",bot);
                getTransferToBots.add(getTransferToBot(bot,transferBotMap.get(bot.getId())));
            });
        }
        getTransferToBotResponse.setUserId(getTransferToBotRequest.getUserId());
        getTransferToBotResponse.setLogin(getTransferToBotRequest.getLogin());
        getTransferToBotResponse.setBotList(getTransferToBots);
        return getTransferToBotResponse;
    }

    public GetTransferToBot getTransferToBot(Bot bot, TransferBot transferBot){
        log.info("bot:{}",bot);
        GetTransferToBot getTransferToBot = new GetTransferToBot();
        getTransferToBot.setBotId(bot.getId());
        getTransferToBot.setBotName(bot.getBotName());
        getTransferToBot.setTransferDate(transferBot.getTransferTime());
        getTransferToBot.setFromlogin(transferBot.getFromLogin());
        getTransferToBot.setTransferId(transferBot.getTransferFromId());
        getTransferToBot.setFromname(api.getUserNameByUid(transferBot.getTransferFromId().toString()));
        return getTransferToBot;
    }

    @Override
//    @Transactional(rollbackFor = Exception.class)
    public void receiveTransferBot(ReceiveTransferBotRequest receiveTransferBotRequest) throws BotException {
        //1.修改transfer_bot表
        List<TransferBot> transferBots = transferBotMapper.selectList(Wrappers.<TransferBot>lambdaQuery()
                .eq(TransferBot::getTransferFromId,receiveTransferBotRequest.getTransferFromId())
                .eq(TransferBot::getTransferToId,receiveTransferBotRequest.getTransferToId())
                .eq(TransferBot::getBotId,receiveTransferBotRequest.getBotId())
                .eq(TransferBot::getIsSuccess,2)
                .orderByDesc(TransferBot::getId));

        TransferBot transferBot = null;

        if (transferBots!=null&&transferBots.size()>0){
            TransactionStatus transactionStatus = null;

            DefaultTransactionDefinition transactionDefinition;
            Bot bot = null;
            RegisterBot registerBot = null;
            try {
                // 有事务，则使用当前事务；否则开启新事务
                transactionDefinition = new DefaultTransactionDefinition();
                transactionDefinition.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
                transactionStatus = transactionManager.getTransaction(transactionDefinition);

                transferBot = transferBots.get(0);
                //转让成功
                transferBot.setIsSuccess(1);
                transferBotMapper.updateById(transferBot);
                //2.将注册表中修改注册者为bot被转让人
                registerBot = registerMapper.selectOne(new QueryWrapper<RegisterBot>()
                        .eq("bot_id",transferBot.getBotId()));
                registerBot.setDeveloperId(receiveTransferBotRequest.getTransferToId());
                registerBot.setDeveloperLogin(receiveTransferBotRequest.getTransferToLogin());
                registerMapper.updateById(registerBot);
                // 3.更改bot表中owner_id字段
                bot = botMapper.selectOne(new QueryWrapper<Bot>()
                        .eq("id",receiveTransferBotRequest.getBotId()));
                bot.setOwnerId(receiveTransferBotRequest.getTransferToId());
                botMapper.updateById(bot);
                // 提交事务
                transactionManager.commit(transactionStatus);
            } catch (Exception e){
                if (transactionStatus != null) {
                    transactionManager.rollback(transactionStatus);
                }
                throw new BotException("Bot注册失败：" + e);
            }
            // 4.bot表中更新client_secret和private_key字段
            if(registerBot!=null&&bot!=null){
                Response response0 = api.activeBotAuth(registerBot.getDeveloperId(), bot.getId(), AuthOperate.UPDATE_SECRET);
                checkException(response0);
                Response response1 = api.activeBotAuth(registerBot.getDeveloperId(), bot.getId(), AuthOperate.UPDATE_PRIVATE_KEY);
                checkException(response1);
            }
        }

    }

    @Override
    public void refuseTransferBot(RefuseTransferBotRequest refuseTransferBotRequest)  throws BotException{
        List<TransferBot> transferBots = transferBotMapper.selectList(Wrappers.<TransferBot>lambdaQuery()
                .eq(TransferBot::getBotId,refuseTransferBotRequest.getBotId())
                .eq(TransferBot::getTransferFromId,refuseTransferBotRequest.getTransferFromBot())
                .eq(TransferBot::getTransferToId,refuseTransferBotRequest.getTransferToBot())
                .eq(TransferBot::getIsSuccess,2)
                .orderByDesc(TransferBot::getId));
        if (transferBots!=null&&transferBots.size()>0){
            TransferBot transferBot = transferBots.get(0);
            transferBot.setIsSuccess(0);
            transferBotMapper.updateById(transferBot);
        }else {
            throw new BotException("不可以拒绝！");
        }
    }

    @Override
    public String getClientId() {
        return "qwer";
    }

    @Override
    public String getClientSecret() {
        return "ert";
    }

    @Override
    public String getBotUrl(String botName) {
        return botName;
    }

    /**
     * 判断该用户是否安装此bot
     * @param repoIds
     * @param botId
     * @return
     */
    @Override
    public JudgeIsInstallBotResponse judgeIsInstallBot(String repoIds, Integer botId) {
        JudgeIsInstallBotResponse judgeIsInstallBotResponse = new JudgeIsInstallBotResponse();
        List<String> installRepoIdList = new ArrayList<>();
        List<String> unInstallRepoIdList = new ArrayList<>();
        List<String> repoIdList = Arrays.asList(repoIds.split(","));
        for (String repoId : repoIdList) {
            //如果公开，则判断安装表里面有没有该用户安装的bot
            List<InstallBot> installBots = installMapper.selectList(Wrappers.
                    <InstallBot>lambdaQuery().eq(InstallBot::getBotId, botId)
                    .eq(InstallBot::getStoreId, repoId));
            if (!Objects.isNull(installBots) && installBots.size() > 0) {
                installRepoIdList.add(repoId);
            } else {
                unInstallRepoIdList.add(repoId);
            }

        }
        judgeIsInstallBotResponse.setInstallRepoIds(installRepoIdList);
        judgeIsInstallBotResponse.setUninstallRepoIds(unInstallRepoIdList);
        return judgeIsInstallBotResponse;

    }

    private boolean judgeLimitIsChange(List<BotLimitEvent> botLimitEvents,List<Limit> limitList){
        if (botLimitEvents.size()!=limitList.size()){
            return true;
        }else {
            List<String> limitEventString = new ArrayList<>();
            botLimitEvents.forEach(botLimitEvent -> {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append(botLimitEvent.getEvent());
                stringBuilder.append(botLimitEvent.getReadWritePr());
                stringBuilder.append(botLimitEvent.getReadWriteCode());
                stringBuilder.append(botLimitEvent.getReadWriteIssue());
                stringBuilder.append(botLimitEvent.getAuthCategory());
                limitEventString.add(stringBuilder.toString());
            });
            List<String> limitString = new ArrayList<>();
            limitList.forEach(limit -> {
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append(limit.getEvent());
                stringBuilder.append(limit.getReadWritePr());
                stringBuilder.append(limit.getReadWriteCode());
                stringBuilder.append(limit.getReadWriteIssue());
                stringBuilder.append(limit.getAuthCategory());
                limitString.add(stringBuilder.toString());
            });
            for (String s : limitString){
                if (!limitEventString.contains(s)){
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 获取用户安装所有bot信息
     * @param userId
     * @param repoIds
     * @return
     */
    @Override
    public GetAllInstallBotsResponse getAllInstallBots(Integer userId, String repoIds) {

        List<InstallBot> installBots = new ArrayList<>();

        if(userId != null) {
            installBots = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                    .eq(InstallBot::getInstallerId,userId));
        }
        if(repoIds != null) {
            List<String> repoList = Arrays.asList(repoIds.split(","));
            installBots = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                    .in(InstallBot::getStoreId, repoList));
        }

        Map<Integer, StringJoiner> botAndAllRepoIdMap = new HashMap<>();
        for(InstallBot installBot : installBots) {
            StringJoiner s = new StringJoiner(",");
            if(botAndAllRepoIdMap.containsKey(installBot.getBotId())){
                s = botAndAllRepoIdMap.get(installBot.getBotId());
            }
            if (installBot.getStoreRepo() == null || (s.length() > 0 && s.toString().contains(installBot.getStoreRepo()))){
                continue;
            }
            botAndAllRepoIdMap.put(installBot.getBotId(), s.add(installBot.getStoreId().toString()));
        }


        List<InstallBotInfo> installBotInfos = new ArrayList<>();
        installBots.forEach(installBot -> {
            Integer botId = installBot.getBotId();
            Bot bot = botMapper.selectOne(Wrappers.<Bot>lambdaQuery()
                    .eq(Bot::getId,botId));
            InstallBotInfo installBotInfo = new InstallBotInfo();
            List<BotLimitEvent> botLimitEvents = botLimitMapper.selectList(Wrappers.<BotLimitEvent>
                    lambdaQuery().eq(BotLimitEvent::getBotId,botId));
            LimitVO limitVO = getLimitFromLimitEvents(botLimitEvents);
            installBotInfo.setLimitVO(limitVO);
            BeanUtils.copyProperties(bot,installBotInfo);
            BeanUtils.copyProperties(installBot,installBotInfo);
            installBotInfo.setStoreRepoIds(String.valueOf(botAndAllRepoIdMap.get(installBotInfo.getBotId())));
            installBotInfos.add(installBotInfo);
        });

        List<InstallBotInfo> installBotInfoList =  installBotInfos.stream().collect(Collectors.collectingAndThen(
                Collectors.toCollection(()->new TreeSet<>(Comparator.comparing(
                        InstallBotInfo::getBotId
                ))),ArrayList::new
        ));
        GetAllInstallBotsResponse getAllInstallBotsResponse = new GetAllInstallBotsResponse();
        getAllInstallBotsResponse.setUserId(userId);
        getAllInstallBotsResponse.setInstallBots(installBotInfoList);
        return getAllInstallBotsResponse;
    }

    /**
     * 获取仓库安装的所有bot
     * @param storeId
     * @return
     */
    @Override
    public GetStoreAllInstallBotResponse getStoreAllInstallBots(Integer storeId) {

        List<InstallBot> installBots = installMapper.selectList(Wrappers.<InstallBot>lambdaQuery()
                .eq(InstallBot::getStoreId,storeId));
        List<InstallBotInfo> installBotInfos = new ArrayList<>();
        installBots.forEach(installBot -> {
            Integer botId = installBot.getBotId();
            Bot bot = botMapper.selectOne(Wrappers.<Bot>lambdaQuery()
                    .eq(Bot::getId,botId));
            InstallBotInfo installBotInfo = new InstallBotInfo();
            List<BotLimitEvent> botLimitEvents = botLimitMapper.selectList(Wrappers.<BotLimitEvent>
                    lambdaQuery().eq(BotLimitEvent::getBotId,botId));
            LimitVO limitVO = getLimitFromLimitEvents(botLimitEvents);
            installBotInfo.setLimitVO(limitVO);
            BeanUtils.copyProperties(bot,installBotInfo);
            BeanUtils.copyProperties(installBot,installBotInfo);
            installBotInfos.add(installBotInfo);
        });

        GetStoreAllInstallBotResponse getStoreAllInstallBotResponse = new GetStoreAllInstallBotResponse();
        getStoreAllInstallBotResponse.setInstallBotInfos(installBotInfos);
        getStoreAllInstallBotResponse.setStoreId(storeId);
        return getStoreAllInstallBotResponse;
    }

    /**
     * 更新bot
     * @param botToMarketRequest
     * @throws BotException
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateMarketBot(BotToMarketRequest botToMarketRequest) throws BotException{
        checkBotName(botToMarketRequest.getBotId(),botToMarketRequest.getMarketName());
        try {
            MarketBot marketBot = marketBotMapper.selectOne(Wrappers
                    .<MarketBot>lambdaQuery().eq(MarketBot::getBotId,botToMarketRequest.getBotId()));
            if (marketBot!=null){
                MarketBot updateMarketBot = getDBMarketBot(botToMarketRequest);
                updateMarketBot.setId(marketBot.getId());
                marketBotMapper.updateById(updateMarketBot);
                //同步更新bot表信息
                Bot bot = botMapper.selectById(marketBot.getBotId());
                bot.setBotDes(updateMarketBot.getMarketIntro());
                bot.setLogo(updateMarketBot.getLogo());
                bot.setBotName(updateMarketBot.getMarketName());
                botMapper.updateById(bot);
            }

        }catch (Exception e){
            throw new BotException("更新失败！");
        }

    }

    /**
     * 下架bot
     * @param botId
     * @throws BotException
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void downMarketBot(Integer botId) throws BotException{
        try {
            MarketBot marketBot = marketBotMapper.selectOne(Wrappers
                    .<MarketBot>lambdaQuery()
                    .eq(MarketBot::getBotId,botId));
            if (!Objects.isNull(marketBot)){
                marketBotMapper.deleteById(marketBot.getId());
            }
        }catch (Exception e){
            throw new BotException("下架bot失败");
        }
    }
}
