package com.gitlink.softbot.service.market;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.gitlink.softbot.global.vo.Response;
import com.gitlink.softbot.utils.RestTemplateUtil;
import com.gitlink.softbot.vo.Webhook;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpMethod;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;

@ActiveProfiles("test")
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT)
class RestTemplateUtilTests {

    private String URL = "https://testforgeplus.trustie.net";

    @Autowired
    private RestTemplate restTemplate;

    private ObjectMapper mapper;

    private Webhook webhooks;

    private Map<String, String> headParams;

    private final static String TOKEN = "eyJraWQiOiJUaEVSLVl3Ukg4TWYwOHM0UnJLUDYzXzZLWmVET2NZckZXcmdzN2VUVWdrIiwiYWxnIjoiSFM1MTIifQ.eyJpc3MiOiJHaXRMaW5rIiwiaWF0IjoxNjc1NzYyMDkwLCJqdGkiOiI0MzA1ZDUwZC01ZGRkLTQ0MzUtODMyNS1iZDczYmVhMWMxYjciLCJ1c2VyIjp7ImlkIjpudWxsLCJsb2dpbiI6bnVsbCwibWFpbCI6bnVsbH19.hpHCJeU4Jyz-DM2NBUdB-tQW_E0-tu9H3LoGhsJ7kPHkSsdXJCII0jxhyPb9gwDsgd8SnlRZF8tZDDjnZSoztQ";


    @BeforeEach
    public void setUp() {
        mapper = new ObjectMapper();
        webhooks = new Webhook(true, "json", "GET", "123456", "http://localhost:10000", "*", new Object[]{"push"}, "softbot");
        headParams = new HashMap<>();
        headParams.put("Authorization", "Bearer "+TOKEN);
    }

    // 测试get params参数调用  以uid获取用户信息接口为例
    @Test
    public void getParams(){
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", "85175");
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.GET, URL+"/api", null, "/users/get_user_info.json", inputParams, null ,headParams);
        //Response response = RestTemplateUtil.getParams(restTemplate,URL +"/users/get_user_info.json",inputParams, headParams);
        System.out.println(response.getData());
    }

    // 测试get PathParams参数调用  以获取用户仓库下webhooks为例
    @Test
    public void getPathParams(){
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", "85175");
        Object[] params = new Object[]{"xxq250", "ruoyi-vue-pro"};
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.GET, URL + "/api/v1", params, "/webhooks.json", inputParams, null ,headParams);

        System.out.println(response.getData());
    }

    // 测试 post 接口调用 以auth_active接口为例
    @Test
    public void postTest(){
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", "84993");
        Object[] params = new Object[]{"800"};
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.POST, URL + "/app", params, "/auth_active", inputParams, null ,headParams);

        System.out.println(response.getData());
    }

    // 测试 post JSon接口调用 以添加仓库webhook接口为例
    @Test
    public void postJson() throws JsonProcessingException {
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", "85175");
        Object[] params = new Object[]{"xxq250", "ruoyi-vue-pro"};
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.POST, URL + "/api/v1", params, "/webhooks.json", inputParams, mapper.writeValueAsString(webhooks) ,headParams);

        System.out.println(response.getData());
    }

    // 测试 patch JSon接口调用 以更新仓库webhook接口为例
    @Test
    public void patchJson() throws JsonProcessingException {
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", "85175");
        Object[] params = new Object[]{"xxq250", "ruoyi-vue-pro"};
        webhooks.setActive(false);
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.PATCH, URL + "/api/v1", params, "/webhooks/1079.json", inputParams, mapper.writeValueAsString(webhooks) ,headParams);

        System.out.println(response.getData());
    }

    // 测试 delete 以删除仓库webhook接口为例
    @Test
    public void deleteTest() throws JsonProcessingException {
        Map<String, String> inputParams = new HashMap<>();
        inputParams.put("uid", "85175");
        Object[] params = new Object[]{"xxq250", "ruoyi-vue-pro"};
        Response response = RestTemplateUtil.httpRequest(restTemplate, HttpMethod.DELETE, URL + "/api/v1", params, "/webhooks/1079.json", inputParams, mapper.writeValueAsString(webhooks) ,headParams);
        System.out.println(response.getData());
    }

}